close all; clear all;
% partition size
tr = .25;

%rand('state',0);

[pde_fig,ax]=pdeinit;
pdetool('appl_cb',1);
set(ax,'DataAspectRatio',[1 1 1]);
set(ax,'PlotBoxAspectRatio',[1.5 1 1]);
set(ax,'XLim',[-1.5 1.5]);
set(ax,'YLim',[-1 1]);
set(ax,'XTickMode','auto');
set(ax,'YTickMode','auto');

% Geometry description:
pdeellip(-0.5,0,0.5,0.5,...
0,'E1');
pdeellip(0.5,0,0.5,0.5,...
0,'E2');
pdeellip(0.5,0,0.3,0.3,...
0,'E3');
pderect([-0.5 0.5 0.3 -0.1],'R2');
pderect([-0.5 0.5 0.5 -0.5],'R1');
set(findobj(get(pde_fig,'Children'),'Tag','PDEEval'),'String','(E1+E2+R1)-(E3+R2)')

% Mesh generation:
setappdata(pde_fig,'trisize',tr);
setappdata(pde_fig,'Hgrad',1.3);
setappdata(pde_fig,'refinemethod','regular');
setappdata(pde_fig,'jiggle',char('on','mean',''));
pdetool('initmesh')

% Plotflags and user data strings:
setappdata(pde_fig,'plotflags',[1 1 1 1 1 1 1 1 0 0 0 1 1 0 0 0 0 1]);
setappdata(pde_fig,'colstring','');
setappdata(pde_fig,'arrowstring','');
setappdata(pde_fig,'deformstring','');
setappdata(pde_fig,'heightstring','');

% export mesh
h=findobj(get(pde_fig,'Children'),'flat','Tag','PDEMeshMenu');
p=get(findobj(get(h,'Children'),'flat','Tag','PDEInitMesh'),...
    'UserData');
e=get(findobj(get(h,'Children'),'flat','Tag','PDERefine'),...
    'UserData');
t=get(findobj(get(h,'Children'),'flat','Tag','PDEMeshParam'),...
    'UserData');

% number of triangles and number of points
M = length(t);
N = length(p);

% find proximity triangles of points and proximity points
T = t(1:3,:);
for j = 1:N
    pos = ceil(find(j == T)./3);
    U{j} = pos';
    NU(j) = length(pos');
    Tp = T(:,pos);
    l = 1; ppo = 0;
    for k = 1:N
        ind = find(Tp == k);
        if length(ind(:)) >= 1
            ppo(l) = k; l = l+1;
        end
    end
    pp{j} = ppo;
end

% calculate laplacian matrix of the graph
L = sparse(N,N);
for i = 1:N
    for j = 1:i
        ppo = pp{i};
        if (i ~= j)&(isempty(find(j == ppo)) == 0)
            L(i,j) = -1;
            L(j,i) = -1;
        elseif (i == j)
            L(i,j) = length(ppo)-1;
        end
    end
end

% calculate second smallest eigenvalue lam and corresponding eigenvector u
[V,D] = eigs(L,2,'SM');
ex = 10.*abs(D(1,1));
%dec = 1; count = 1;
%while (dec)
%    lam = D(count,count);
%    if abs(lam) >= ex        
%        dec = 0;
%        u = V(:,count);
%    end
%    count = count + 1;
%end

lam = D(1,1); u = V(:,1);

%calculate median of all components of u
m_u = median(u);

% calculate bisection
nu_1 = []; nu_2 = [];
for i = 1:N
    if u(i) < m_u
        nu_1 = [nu_1,i];
    elseif u(i) > m_u
        nu_2 = [nu_2,i];
    else
        dec = rand(1);
        if dec >= .5
            nu_1 = [nu_1,i];
        else
            nu_2 = [nu_2,i];
        end
    end
end


% plot subgraphs
hold on;
for i = nu_1
    ppo = pp{i};
    for j = nu_1
        ind = find(j == ppo);
        if ~isempty(ind)
            plot([p(1,i) p(1,j)],[p(2,i) p(2,j)],'-k');
        end
    end
    for j = nu_2
        ind = find(j == ppo);
        if ~isempty(ind)
            plot([p(1,i) p(1,j)],[p(2,i) p(2,j)],'-r','Linewidth',2);
        end
    end
end

for i = nu_2
    ppo = pp{i};
    for j = nu_2
        ind = find(j == ppo);
        if ~isempty(ind)
            plot([p(1,i) p(1,j)],[p(2,i) p(2,j)],'-k');
        end
    end
end
plot(p(1,nu_1),p(2,nu_1),'o','MarkerEdgeColor',[0 .5 0],'MarkerFaceColor',[0 .5 0]);
plot(p(1,nu_2),p(2,nu_2),'o','MarkerEdgeColor','b','MarkerFaceColor','b');

    
% calculate size of partitions, cut size and lower bound for cut size
disp('Groesse der grnen Partition: ')
gnu_1 = length(nu_1);
disp(gnu_1)
disp('Groesse der blauen Partition: ')
gnu_2 = length(nu_2);
disp(gnu_2)

cutsize = 0;
for i = nu_1
    ppo = pp{i};
    for j = ppo
        dec = find(j == nu_2);
        if length(dec) >= 1
            cutsize = cutsize + 1;
        end
    end
end
disp('Schnittkosten: ')
disp(cutsize)

disp('untere Grenze fr Schnittkosten:')
disp((1./4).*lam.*N)    